@@ -44,6 +44,12 @@
   "Helm eshell completion and history."
   :group 'helm)
 
+
+(defcustom helm-eshell-fuzzy-match nil
+  "Enable fuzzy matching in `helm-esh-pcomplete' when non--nil."
+  :group 'helm-eshell
+  :type 'boolean)
+
 
 (defvar helm-eshell-history-map
   (let ((map (make-sparse-keymap)))
@@ -222,7 +228,8 @@ The function that call this should set `helm-ec-target' to thing at point."
                          (car (last (ignore-errors
                                       (pcomplete-parse-arguments))))))
              (with-helm-show-completion beg end
-               (or (helm :sources (helm-make-source "Eshell completions" 'helm-esh-source)
+               (or (helm :sources (helm-make-source "Eshell completions" 'helm-esh-source
+                                    :fuzzy-match helm-eshell-fuzzy-match)
                          :buffer "*helm pcomplete*"
                          :keymap helm-esh-completion-map
                          :resume 'noresume
