@@ -632,6 +632,7 @@ If N is positive go forward otherwise go backward."
                (goto-char it)
              (forward-line 1))
            (funcall mark-maybe)))
+    (helm-follow-execute-persistent-action-maybe)
     (helm-log-run-hook 'helm-move-selection-after-hook)))
 
 ;;;###autoload
@@ -900,6 +901,13 @@ These extensions will be added to command line with --include arg of grep."
    (keymap :initform helm-grep-map)
    (nohighlight :initform t)
    (nomark :initform t)
+   (backend :initarg :backend
+            :initform nil
+            :documentation
+            "  The grep backend that will be used.
+  It is actually used only as an internal flag
+  and don't set the backend by itself.
+  You probably don't want to modify this.")
    (candidate-number-limit :initform 9999)
    (help-message :initform 'helm-grep-help-message)
    (history :initform 'helm-grep-history)
@@ -910,11 +918,21 @@ These extensions will be added to command line with --include arg of grep."
 
 (defvar helm-source-grep nil)
 
-(defun helm-do-grep-1 (targets &optional recurse grep exts default-input input)
-  "Launch grep on a list of TARGETS files.
+(defmethod helm--setup-source ((source helm-grep-class))
+  (call-next-method)
+  (helm-aif (and helm-follow-mode-persistent
+                 (if (eq (slot-value source 'backend) 'git)
+                     helm-source-grep-git
+                     helm-source-grep))
+      (setf (slot-value source 'follow)
+            (assoc-default 'follow it))))
 
-When RECURSE is given use -r option of grep and prompt user
-for EXTS to set the --include args of grep.
+(cl-defun helm-do-grep-1 (targets &optional recurse backend exts
+                                  default-input input (source 'helm-source-grep))
+  "Launch helm using backend BACKEND on a list of TARGETS files.
+
+When RECURSE is given and BACKEND is 'grep' use -r option of
+BACKEND and prompt user for EXTS to set the --include args of BACKEND.
 Interactively you can give more than one arg separated by space at prompt.
 e.g
     $Pattern: *.el *.py *.tex
@@ -927,9 +945,9 @@ If prompt is empty `helm-grep-ignored-files' are added to --exclude.
 Argument DEFAULT-INPUT is use as `default' arg of `helm' and INPUT
 is used as `input' arg of `helm', See `helm' docstring.
 
-Arg GREP when non--nil specify which grep to use
-It is used actually to specify 'zgrep or 'git.
-When zgrep is used don't prompt for a choice
+Arg BACKEND when non--nil specify which backend to use
+It is used actually to specify 'zgrep' or 'git'.
+When BACKEND 'zgrep' is used don't prompt for a choice
 in recurse, and ignore EXTS, search being made recursively on files matching
 `helm-zgrep-file-extension-regexp' only."
   (when (and (helm-grep-use-ack-p)
@@ -939,7 +957,7 @@ in recurse, and ignore EXTS, search being made recursively on files matching
   (let* (non-essential
          (exts (and recurse
                     ;; [FIXME] I could handle this from helm-walk-directory.
-                    (not (eq grep 'zgrep)) ; zgrep doesn't handle -r opt.
+                    (not (eq backend 'zgrep)) ; zgrep doesn't handle -r opt.
                     (not (helm-grep-use-ack-p :where 'recursive))
                     (or exts (helm-grep-get-file-extensions targets))))
          (include-files
@@ -951,16 +969,14 @@ in recurse, and ignore EXTS, search being made recursively on files matching
                               (remove "*" exts)
                               exts) " ")))
          (types (and (not include-files)
-                     (not (eq grep 'zgrep))
+                     (not (eq backend 'zgrep))
                      recurse
                      (helm-grep-use-ack-p :where 'recursive)
                      ;; When %e format spec is not specified
                      ;; ignore types and do not prompt for choice.
                      (string-match "%e" helm-grep-default-command)
                      (helm-grep-read-ack-type)))
-         (follow (and helm-follow-mode-persistent
-                      (assoc-default 'follow helm-source-grep)))
-         (src-name (capitalize (helm-grep-command recurse grep))))
+         (src-name (capitalize (helm-grep-command recurse backend))))
     ;; When called as action from an other source e.g *-find-files
     ;; we have to kill action buffer.
     (when (get-buffer helm-action-buffer)
@@ -973,14 +989,14 @@ in recurse, and ignore EXTS, search being made recursively on files matching
     ;; We need to store these vars locally
     ;; to pass infos later to `helm-resume'.
     (helm-set-local-variable
-     'helm-zgrep-recurse-flag (and recurse (eq grep 'zgrep))
+     'helm-zgrep-recurse-flag (and recurse (eq backend 'zgrep))
      'helm-grep-last-targets targets
      'helm-grep-include-files (or include-files types)
      'helm-grep-in-recurse recurse
-     'helm-grep-use-zgrep (eq grep 'zgrep)
+     'helm-grep-use-zgrep (eq backend 'zgrep)
      'helm-grep-default-command
-     (cond ((eq grep 'zgrep) helm-default-zgrep-command)
-           ((eq grep 'git) helm-grep-git-grep-command)
+     (cond ((eq backend 'zgrep) helm-default-zgrep-command)
+           ((eq backend 'git) helm-grep-git-grep-command)
            (recurse helm-grep-default-recurse-command)
            ;; When resuming, the local value of
            ;; `helm-grep-default-command' is used, only git-grep
@@ -988,11 +1004,11 @@ in recurse, and ignore EXTS, search being made recursively on files matching
            (t helm-grep-default-command))
      'default-directory helm-ff-default-directory) ;; [1]
     ;; Setup the source.
-    (setq helm-source-grep (helm-make-source src-name 'helm-grep-class
-                             :follow follow))
+    (set source (helm-make-source src-name 'helm-grep-class
+                  :backend backend))
     (helm
-     :sources 'helm-source-grep
-     :buffer (format "*helm %s*" (helm-grep-command recurse grep))
+     :sources source
+     :buffer (format "*helm %s*" (helm-grep-command recurse backend))
      :default default-input
      :input input
      :keymap helm-grep-map
@@ -1354,6 +1370,13 @@ if available with current AG version."
 
 (defvar helm-source-grep-ag nil)
 
+(defmethod helm--setup-source ((source helm-grep-ag-class))
+  (call-next-method)
+  (helm-aif (and helm-follow-mode-persistent
+                 helm-source-grep-ag
+                 (assoc-default 'follow helm-source-grep-ag))
+      (setf (slot-value source 'follow) it)))
+
 (defun helm-grep-ag-1 (directory &optional type)
   "Start helm ag in DIRECTORY maybe searching in files of type TYPE."
   (setq helm-source-grep-ag
@@ -1384,6 +1407,8 @@ When WITH-TYPES is non-nil provide completion on AG types."
 ;;; Git grep
 ;;
 ;;
+(defvar helm-source-grep-git nil)
+
 (defcustom helm-grep-git-grep-command
   "git --no-pager grep -n%cH --color=always --exclude-standard --no-index --full-name -e %p -- %f"
   "The git grep default command line.
@@ -1415,7 +1440,7 @@ arg INPUT is what you will have by default at prompt on startup."
          (helm-ff-default-directory (funcall helm-grep-default-directory-fn)))
     (cl-assert helm-ff-default-directory nil "Not inside a Git repository")
     (helm-do-grep-1 (if all '("") `(,(expand-file-name directory)))
-                    nil 'git nil default input)))
+                    nil 'git nil default input 'helm-source-grep-git)))
 
 
 ;;;###autoload
@@ -1424,7 +1449,7 @@ arg INPUT is what you will have by default at prompt on startup."
 With prefix-arg prompt for type if available with your AG version."
   (interactive "P")
   (require 'helm-files)
-  (helm-grep-ag default-directory arg))
+  (helm-grep-ag (expand-file-name default-directory) arg))
 
 ;;;###autoload
 (defun helm-grep-do-git-grep (arg)
